/*http://lejos-osek.sourceforge.net/nxtway_gs.htm*/
#include "kernel.h"
#include "kernel_id.h"
#include "ecrobot_interface.h"

#include "balancer.h" /* NXTway-GS C API header file  <-This header file has to be included*/
#include "nxt_config.h"

typedef enum {
  INIT_MODE,   /* system initialize mode */
  CAL_MODE,    /* gyro sensor offset calibration mode */
  CONTROL_MODE /* balance and RC control mode */
} MODE;

U16 current_color = 0;

#define MIN_DISTANCE (25)    /* minimum distance in cm for obstacle avoidance */
#define BT_RCV_BUF_SIZE (32) /* it must be 32bytes with NXT GamePad */

MODE nxtway_gs_mode = INIT_MODE; /* NXTway-GS mode */
volatile U8 obstacle_flag;       /* 1(obstacle detected)/0(no obstacle) */

void ecrobot_device_initialize(void)
{
  ecrobot_init_sonar_sensor(PORT_SONAR);
  ecrobot_init_nxtcolorsensor(LIGHT, NXT_LIGHTSENSOR_RED);
}

void ecrobot_device_terminate(void)
{
  nxt_motor_set_speed(PORT_MOTOR_L, 0, 1);
  nxt_motor_set_speed(PORT_MOTOR_R, 0, 1);
  ecrobot_term_sonar_sensor(PORT_SONAR);
  ecrobot_term_bt_connection();
}

DeclareCounter(SysTimerCnt);

void user_1ms_isr_type2(void)
{
  /* Increment System Timer Count to activate periodical Tasks */
  (void)SignalCounter(SysTimerCnt);
}

U32 LIGHT_THRESHOLD = 320;
/*===================================BALANCING=========================================== */
TASK(BALANCING)
{
  static U32 gyro_offset;    /* gyro sensor offset value */
  static U32 avg_cnt;        /* average counter to calc gyro offset */
  static U32 cal_start_time; /* calibration start time */
  static U8 bt_receive_buf[BT_RCV_BUF_SIZE]; /* Bluetooth receive buffer(32bytes) */
  SINT i;
  S8 cmd_forward,cmd_turn;
  S8 pwm_l,pwm_r;

  switch(nxtway_gs_mode){
    case(INIT_MODE):
      gyro_offset = 0;
      avg_cnt = 0;
      for (i = 0; i < BT_RCV_BUF_SIZE; i++){
        bt_receive_buf[i] = 0;
      }
      balance_init(); /* NXTway-GS C API initialize function <-Call balance init function*/
      nxt_motor_set_count(PORT_MOTOR_L, 0); /* reset left motor count */
      nxt_motor_set_count(PORT_MOTOR_R, 0); /* reset right motor count */
      cal_start_time = ecrobot_get_systick_ms();
      nxtway_gs_mode = CAL_MODE;
      break;

    case(CAL_MODE):
      gyro_offset += (U32)ecrobot_get_gyro_sensor(PORT_GYRO);
      avg_cnt++;
      /* 1000msec later, start balancing */
      if ((ecrobot_get_systick_ms() - cal_start_time) >= 1000U){
        gyro_offset /= avg_cnt;
        ecrobot_sound_tone(440U, 500U, 30); /* beep a tone */
        nxtway_gs_mode =  CONTROL_MODE;
      }
      break;

    case(CONTROL_MODE):
        // make NXTway-GS move forward, turn, and detect white surfaces 
   	if (current_color <= LIGHT_THRESHOLD) {
	      cmd_forward= 20;
	      cmd_turn = 0;
 	}
	else {
	       cmd_forward= 30;
	       cmd_turn = 30;
   	}

        // make NXTway-GS move backward to avoid obstacle 
      if (obstacle_flag == 1){
        cmd_forward = -100;
        cmd_turn = 0;
      }

	
      /* NXTway-GS C API balance control function (has to be invoked in 4msec period) */
      balance_control( //<-Call balance control function
        (F32)cmd_forward,
        (F32)cmd_turn,
        (F32)ecrobot_get_gyro_sensor(PORT_GYRO),
        (F32)gyro_offset,
        (F32)nxt_motor_get_count(PORT_MOTOR_L),
        (F32)nxt_motor_get_count(PORT_MOTOR_R),
       (F32)ecrobot_get_battery_voltage(),
        &pwm_l,
        &pwm_r);
      nxt_motor_set_speed(PORT_MOTOR_L, pwm_l, 1);
      nxt_motor_set_speed(PORT_MOTOR_R, pwm_r, 1);

      //ecrobot_bt_data_logger(cmd_forward, cmd_turn); /* Bluetooth data logger */
      break;

    default:
      /* unexpected mode */
      nxt_motor_set_speed(PORT_MOTOR_L, 0, 1);
      nxt_motor_set_speed(PORT_MOTOR_R, 0, 1);
      break;
  }

  TerminateTask(); /* terminates this task and executed by System Timer Count */
}

/*===================================SONARSensor=========================================== */
TASK(SONARSensor)
{
  obstacle_flag = 0; /* no obstacles */
   if (ecrobot_get_sonar_sensor(PORT_SONAR) <= MIN_DISTANCE){
     obstacle_flag = 1; /* obstacle detected */
  }

  TerminateTask(); /* terminates this task and executed by System Timer Count */
}

void show_light(void)
{
  display_goto_xy(0, 2);	
  display_int(current_color, 6);
}

void read_sensor(void)
{
  ecrobot_process_bg_nxtcolorsensor(); 
  current_color = ecrobot_get_nxtcolorsensor_light(LIGHT); 
}



/*===================================LCD=========================================== */
TASK(LCDMonitor)
{
  display_clear(1);
  show_light();
  display_update();
  TerminateTask();
}

/*==================================LIGHT========================================== */
TASK(LIGHTSensor)
{
  read_sensor();
  TerminateTask();
}

