/* rover.c */ 

#include "kernel.h"
#include "kernel_id.h"
#include "ecrobot_interface.h"
#include "string.h"
/* own definitions */
#include "config.h"



/* OSEK declarations */

DeclareCounter(SysTimerCnt);
DeclareTask(LIGHTSensor);
DeclareTask(LCDMonitor);
DeclareTask(MOTORsensor);
DeclareTask(CALIBRATION);


/* global variables  */

CHAR status[8];
U32 counter = 0;

U32 LIGHT_THRESHOLD = 360;
/* Global vars:*/
    int offset = 520;
    int prev_error = 0;
    float integral_error = 0; 
    int scale = 20;
    U16 current_color_rear = 0;
    U16 current_color_front = 0;

U32 itoa(char status[], int len, int n)
{
  if (n == 0) {
    status[0] = '0';
    status[1] = 0;
    return 1;
  } 

    U32 i = 0;
    while (n && i < len - 1) {
      status[i] = n % 10 + '0';
      n /= 10;
      i ++;
    }
    status[i] = 0;

    U32 end = i;
    
    U32 j = 0;
    i --;
    while (j < i) {
      CHAR c = status[i];
      status[i] = status[j];
      status[j] = c;
      j ++;
      i --;
    }
    return end;
}

void show_light(void)
{
  display_goto_xy(0, 2);	
  display_int(current_color_front, 6);
}

void show_light2(void)
{
  display_goto_xy(0, 4);	
  display_int(current_color_rear, 6);
}

void read_sensor(void)
{

  ecrobot_process_bg_nxtcolorsensor(); 
  current_color_front = ecrobot_get_nxtcolorsensor_light(LIGHT); 
  current_color_rear = ecrobot_get_nxtcolorsensor_light(LIGHT2); 
}


/*Works everywhere w/o recompiling the program*/
int calibration =0; /*User Calibration*/
void doCalibration(void)
{
   /* wait till user click the orange button - user balance it */
  while(!calibration){
	U8 rs=0;
	while (!rs)
	{
		read_sensor();
		show_light();
		rs= ecrobot_is_ENTER_button_pressed();
		offset = current_color_front;
	}	
	calibration =1;	
   }
}

/*http://www.inpharmix.com/jps/PID_Controller_For_Lego_Mindstorms_Robots.html*/
/* PID constants */
    int Kp = 25; /*the Konstant for the proportional controller - Kp*(error) */
    int Ki = 5;  /*multiply the integral by  the constant (Ki) to get a correction - Ki*(integral) */
    int Kd = 40; /*predicting the future*/

void change_motor(){
    
    /* Proportional Error:*/
    int error = current_color_front - offset;

    /* Integral Error:*/
    integral_error = ((integral_error + error) * 2)/3;
             
    /* Derivative Error: */
    int derivative_error = error - prev_error;
    
    prev_error = error;
             
    int pid = (int)(Kp * error + Ki * integral_error + Kd * derivative_error) / scale;
             
    if (pid > 100)
        pid = 100;
    if (pid < -100)
        pid = -100;
 
    /* Power derived from PID value:
     abs (pid)*/
    int power = pid * ( (pid<0) * (-1) + (pid>0)); 
    power = 55 + (power * 45) / 100; // NORMALIZE POWER
 
    if (pid > 0){ /*FORWARD*/
	 nxt_motor_set_speed(RIGHT_MOTOR, power, 1);
	 nxt_motor_set_speed(LEFT_MOTOR,  power, 1);
    } else { /*BACKWARD*/
	 nxt_motor_set_speed(RIGHT_MOTOR, -power, 1);
	 nxt_motor_set_speed(LEFT_MOTOR,  -power, 1);
    } 

}


/* LEJOS OSEK hooks */

void ecrobot_device_initialize()

{
  ecrobot_init_nxtcolorsensor(LIGHT, NXT_LIGHTSENSOR_RED);
  ecrobot_init_nxtcolorsensor(LIGHT2, NXT_LIGHTSENSOR_RED);
  /* ecrobot_init_sonar_sensor(SONIC);   */
  nxt_motor_set_speed(RIGHT_MOTOR, 0, 1); 
  nxt_motor_set_speed(LEFT_MOTOR, 0, 1);
}

void ecrobot_device_terminate()

{
  ecrobot_term_nxtcolorsensor(LIGHT);
  ecrobot_term_nxtcolorsensor(LIGHT2);
  /* ecrobot_term_sonar_sensor(SONIC); */
  nxt_motor_set_speed(RIGHT_MOTOR, 0, 1); 
  nxt_motor_set_speed(LEFT_MOTOR, 0, 1); 
}

/* nxtOSEK hook to be invoked from an ISR in category 2 */
void user_1ms_isr_type2(void)
{
  StatusType ercd;

  ercd = SignalCounter(SysTimerCnt); /* Increment OSEK Alarm Counter */
  if (ercd != E_OK)
  {
    ShutdownOS(ercd);
  }
}

/***********************************TASKS****************************************** */

/*==================================MOTOR========================================== */
TASK(MOTORSensor)
{
  change_motor();
  TerminateTask();
}

/*===================================LCD=========================================== */
TASK(LCDMonitor)
{
  display_clear(1);
  show_light();
  show_light2();
  display_update();
  TerminateTask();
}

/*==================================LIGHT========================================== */
TASK(LIGHTSensor)
{
  read_sensor();
  TerminateTask();
}
/*================================CALIBRATION====================================== */
TASK(CALIBRATION)
{
  doCalibration();
  TerminateTask();
}
/******************************** END OF FILE ********************************/



